#!/usr/bin/perl -w
use strict;
use feature qw/say/;

=head1 NAME

photofolder.pl - the preforked server of PhotoFolder

=head1 SYNOPSIS

    photofolder.pl [options] [LSB_COMMAND]

    photofolder.pl -p /tmp/photofolder.pid status

=head1 OPTIONS

=over 4

=item B<-A TCP_ADDR, --addr=TCP_ADDR>

Sets the TCP address of listener. Default: "*"

=item B<-d, --debug>

Prints debug information on STDOUT and switch to off daemonize of the worker!

=item B<-D DIR, --documentroot=DIR>

Use DIR as DocumentRoot directory

=item B<-p PIDFILE, --pid=PIDFILE>

Sets PID file location

=item B<-P TCP_PORT, --port=TCP_PORT>

Sets the TCP port of listener. Default: 8082

=item B<-S DIR, --sharedir=DIR, --homedir=DIR>

Use DIR as DocumentRoot directory

=back

=head1 DESCRIPTION

This tool provides running of the PhotoFolder preforked server

=head1 AUTHOR

Serz Minus (Sergey Lepenkov) L<http://www.serzik.com> E<lt>minus@serzik.comE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2021 D&D Corporation. All Rights Reserved

=head1 LICENSE

Copyright 2021 Serz Minus (Sergey Lepenkov)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

See C<LICENSE> file or L<https://www.apache.org/licenses/LICENSE-2.0.txt>

=cut

use Getopt::Long;
use Pod::Usage;

use PhotoFolder;
use Mojo::Server::Prefork;

use constant {
    DEBUGMODE      => 0,
    PID_FILE       => "/tmp/photofolder.pid",
    MAX_CLIENTS    => 100, # See Mojo::Server::Prefork
    MAX_REQUESTS   => 100, # See Mojo::Server::Prefork
    ACCEPTS        => 0, # See Mojo::Server::Prefork
    SPARE          => 2, # See Mojo::Server::Prefork
    WORKERS        => 4, # See Mojo::Server::Prefork
    LISTEN         => 'http://%s:%s', # See Mojo::Server::Prefork
    DOCUMENTROOT   => "/var/www/html", # Photo location
    HOMEDIR        => undef, # Trmplates and static data location
    CMDDEFAULT     => "status",
    DEFAULT_PORT   => 8082,
    DEFAULT_ADDR   => "*",
};

my $options = {};
Getopt::Long::Configure ("bundling");
GetOptions($options,
    # Information and debug
    "help|usage|h",         # Show help page
    "longhelp|H|?",         # Show long help page
    "debug|d",              # Debug mode
    "kill|k=s",             # Kill (signal name)

    # Application
    "documentroot|dir|D=s", # DocumentRoot
    "homedir|sharedir|S=s", # HomeDir
    "pidfile|pid|p=s",      # PID file
    "port|P=s",             # TCP port
    "addr|A=s",             # TCP addr

) || pod2usage(-exitval => 1, -verbose => 0, -output => \*STDERR);
pod2usage(-exitval => 0, -verbose => 1) if $options->{help};
pod2usage(-exitval => 0, -verbose => 2) if $options->{longhelp};

my $dash_k = shift(@ARGV) || CMDDEFAULT;

my $app = PhotoFolder->new(
        debugmode      => $options->{debug} || DEBUGMODE,
        documentroot   => $options->{documentroot} || DOCUMENTROOT,
        homedir        => $options->{homedir} || HOMEDIR,
    );
$ENV{MOJO_MODE} ||= 'production' unless $app->debugmode;

my $prefork = Mojo::Server::Prefork->new( app => $app ); # app => $self
    $prefork->pid_file($options->{pidfile} || PID_FILE);
    $prefork->max_clients(MAX_CLIENTS);
    $prefork->max_requests(MAX_REQUESTS);
    $prefork->accepts(ACCEPTS);
    $prefork->spare(SPARE);
    $prefork->workers(WORKERS);
    $prefork->listen([sprintf(LISTEN, $options->{addr} || DEFAULT_ADDR, $options->{port} || DEFAULT_PORT)]);
my $pid = $prefork->check_pid();

# LSB start, stop, status
if ($dash_k eq 'start') {
    if ($pid) {
        say "Already running $pid";
        exit 0;
    }
} elsif ($dash_k eq 'stop') {
    if ($pid) {
        kill 'QUIT', $pid;
        say "Stopping $pid";
        exit 0;
    } else {
        say "Not running";
    }
    exit 0;
} elsif ($dash_k eq 'status') {
    say $pid ? "Running $pid" : "Not running";
    exit 0;
} else {
    say STDERR "Incorrect LSB command";
    exit 1;
}

# Daemonize
$prefork->daemonize() unless $app->debugmode();

print "Running\n";
$prefork->run;

1;

__END__
